"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assignableObjectPairs = void 0;
const tsutils_1 = require("tsutils");
/**
 * Throws away non-object types (string, number, boolean, etc) because we don't check those for readonly -> mutable assignment
 * and returns an array of pairs of types that are assignable.
 */
exports.assignableObjectPairs = (rawDestinationType, rawSourceType, checker) => {
    const isAssignableTo = checker.isTypeAssignableTo;
    // eslint-disable-next-line functional/no-conditional-statement
    if (isAssignableTo === undefined) {
        return [];
    }
    const destinationTypeParts = tsutils_1.unionTypeParts(rawDestinationType).filter((t) => tsutils_1.intersectionTypeParts(t).every((itp) => tsutils_1.isObjectType(itp)));
    const sourceTypeParts = tsutils_1.unionTypeParts(rawSourceType).filter((t) => tsutils_1.intersectionTypeParts(t).every((itp) => tsutils_1.isObjectType(itp)));
    return sourceTypeParts.flatMap((sourceTypePart) => destinationTypeParts
        .filter((destinationTypePart) => isAssignableTo(sourceTypePart, destinationTypePart))
        .map((destinationTypePart) => ({
        sourceType: sourceTypePart,
        destinationType: destinationTypePart,
    })));
};
