"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
/**
 * An ESLint rule to enforce TypeScript strict mode.
 */
const requireStrictMode = {
    meta: {
        type: "problem",
        docs: {
            category: "Possible Errors",
            description: "Enforces the use of TypeScript's strict mode.",
            recommended: "error",
            url: "https://github.com/danielnixon/eslint-plugin-total-functions",
        },
        messages: {
            errorStringGeneric: "TypeScript's strict mode is required.",
        },
        schema: [],
    },
    create: (context) => {
        const parserServices = experimental_utils_1.ESLintUtils.getParserServices(context);
        return {
            Program: (node) => {
                // eslint-disable-next-line functional/no-conditional-statement
                if (parserServices.program.getCompilerOptions().strict !== true) {
                    // eslint-disable-next-line functional/no-expression-statement
                    context.report({
                        node: node,
                        messageId: "errorStringGeneric",
                    });
                }
            },
        };
    },
};
exports.default = requireStrictMode;
